<?php

function htmlEscape($str)
{
	return htmlspecialchars($str, ENT_QUOTES, 'UTF-8');
}

function parseCond(&$data, $key)
{
	if (isset($data[$key]))
	{
		$var = $data[$key];
		return  (is_array($var) && count($var)) ||
	    	   (!is_array($var) && $var);
	}
	else
		return FALSE;
}

function parseTemplate($fileTpl, &$data = NULL)
{
	$filePhp = substr($fileTpl, 0, -4) . '.php';

	// filemtime returns file modification time or FALSE on error
	$p = @filemtime($filePhp);
	$t = @filemtime($fileTpl);

	// PHP file is absent or obsolete
	if ($p === FALSE || $p < $t)
	{
		// template is NOT php-script, read it as plain text
		$text = file_get_contents($fileTpl);

		// then if data provided, substitute placeholders
		if (is_array($data) && count($data))
		{
			// split to parts divided by "{ key }" or "<!-- key -->"
			$parts = preg_split('/(\{.*\})|(<!--.*-->)/U', $text, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
			if (count($parts) > 1)
			{
				$text = '';
				foreach ($parts as $clause)
				{
					if ($clause{0} == '{' || substr($clause, 0, 4) == '<!--')
					{
						$key = trim(($clause{0} == '{') ? substr($clause, 1, -1) : substr($clause, 4, -3));

						if ($key{0} == '%')
						{
							if ($key == '%END')
								$clause = '<?php } ?>';
							else if (substr($key, 0, 3) == '%IF')
								$clause = '<?php if (parseCond($data, \''.trim(substr($key, 3)).'\')) { ?>';
							else if ($key == '%ELSE')
								$clause = '<?php } else { ?>';
							else if (substr($key, 0, 8) == '%FOREACH')
							{
								list($arrname, $itemname) = array_map('trim', explode('AS', substr($key, 8)));
								$clause = '<?php foreach ($data[\'' . $arrname . '\'] as $data[\'' . $itemname . '\']) { ?>';
							}
							// recursively parse sub-templates
							else if (substr($key, 0, 8) == '%INCLUDE')
							{
								$fileSubTpl = dirname($fileTpl) . '/' . trim(substr($key, 8)) . '.tpl';
								$clause = '<?php parseTemplate(\'' . $fileSubTpl . '\', $data); ?>';
							}
						}
						else
						{
							$escape = ($key{0} == '@');

							// it is wise to escape user data
							if ($escape)
								$key = substr($key, 1);

							// (multidimentional) array item
							if (strpos($key, ':') !== FALSE)
								$clause = '$data[\'' . implode('\'][\'', explode(':', $key)) . '\']';
							// or simple var
							else
								$clause = '$data[\''.$key.'\']';

							$clause = ($escape) ?
								('<?php echo htmlEscape(' . $clause . '); ?>') :
								('<?php echo ' . $clause . '; ?>');
						}
					}

					$text .= $clause;
				}
			}
		}

		// store parsed result as PHP file
		file_put_contents($filePhp, $text);

	} // PHP file absent or obsolete

	include $filePhp;
}